from enum import Enum
import numpy as np
import matplotlib.pyplot as plt

class SideOrder(Enum):
    BUY = 1
    SELL = 2

class Order():
    def __init__(self, side, price):
        assert isinstance(side, SideOrder), "ORDER ERROR: SIDE IS NOT SIDEORDER"
        self.price = price
        self.side = side

class Trade():
    def __init__(self, entryorder, exitorder):
        assert entryorder.side != exitorder.side, "TRADE ERROR: SAME SIDE ORDERS"
        self.entry_order = entryorder
        self.exit_order = exitorder

class TradeHistory():
    def __init__(self, trades):
        self.trades = trades
    
    def study(self, cash=1000.0, sizer=1.0, commision=0.0, show_plot=True):
        assert 0 <= commision and commision <= 1, "Comission must be number between 0 and 1"
        assert 0 <= sizer and sizer <= 1, "Sizer must be number between 0 and 1"
        assert 0 < cash, "Cash must be number between 0 and 1"
        
        history_cash = [cash]
        for trade in self.trades:
            amount_order = sizer*cash
            cash -= amount_order
            amount_order -= (amount_order*commision)
            pnl = trade.exit_order.price - trade.entry_order.price
            if trade.entry_order.side == SideOrder.SELL:
                pnl *= -1
            amount_order += pnl
            amount_order -= (amount_order*commision)
            cash += amount_order
            history_cash.append(cash)
        
        if show_plot:
            t = np.arange(0, len(history_cash))
            fig, ax = plt.subplots()
            ax.plot(t, history_cash)
            ax.set(xlabel='time', ylabel='price units',
                title='Portfolio History')
            ax.grid()
            plt.show()

        return cash





import pandas as pd
from pandas.api.types import is_numeric_dtype, is_datetime64_any_dtype
from .trades import Order, Trade, SideOrder

columns_supported = ['datetime', 'open', 'high', 'low', 'close', 'volume']

class Backtesting():

    def __init__(self):
        self.data = None
        self.trades = []
        self.entryorder = None
        self.exitorder = None
        self.candles = None

    def setdata(self, data):
        assert isinstance(data, pd.DataFrame), "DATA must be instance of pandas Dataframe"
        assert set(columns_supported).issubset(data.columns), "data columns must be ['datetime', 'open', 'high', 'low', 'close', 'volume']"
        
        assert is_datetime64_any_dtype(data['datetime']), "Datetime Column not is datetime64"
        assert is_numeric_dtype(data['open']), "Open Column not is numeric"
        assert is_numeric_dtype(data['high']), "High Column not is numeric"
        assert is_numeric_dtype(data['low']), "Low Column not is numeric"
        assert is_numeric_dtype(data['close']), "Close Column not is numeric"
        assert is_numeric_dtype(data['volume']), "Volume Column not is numeric"

        data.sort_values(by=['datetime'], inplace=True)
        data.set_index('datetime', inplace=True)

        self.data = data
    
    def setdatafromcsv(self, path):
        self.setdata(pd.read_csv(path))
    
    def savetrade(self):
        if self.entryorder and self.exitorder:
            self.trades.append(Trade(self.entryorder, self.exitorder))
            self.entryorder = None
            self.exitorder = None

    def buy(self):
        price = self.candles['close'][-1]
        if self.entryorder == None:
            self.entryorder = Order(SideOrder.BUY, price)
        else:
            self.exitorder = Order(SideOrder.BUY, price)

        self.savetrade()
        return True

    def sell(self):
        price = self.candles['close'][-1]
        if self.entryorder == None:
            self.entryorder = Order(SideOrder.SELL, price)
        else:
            self.exitorder = Order(SideOrder.SELL, price)

        self.savetrade()
        return True

    def next(self):
        pass

    def run(self):
        assert isinstance(self.data, pd.DataFrame), "DATA MUST BE DATAFRAME"

        for x in range(1, len(self.data)):
            # print(self.data[:x])
            self.candles = pd.DataFrame(self.data[:x])
            self.next()
        
        if self.entryorder:
            if not self.exitorder:
                if self.entryorder.side == SideOrder.BUY:
                    self.exitorder = Order(SideOrder.SELL, self.candles['close'][-1])
                elif self.entryorder.side == SideOrder.SELL:
                    self.exitorder = Order(SideOrder.BUY, self.candles['close'][-1])
                else:
                    raise Exception("RUN SIDE UNKNOWN")
                self.savetrade()
        
        return self.trades

        

    

        
        
        
    



from nanobt.backtesting import Backtesting
from nanobt.trades import TradeHistory
import pandas as pd

INIT_PORTFOLIO = 1000
class BuyAndHoldStrategy(Backtesting):
    def __init__(self):
        super().__init__()
        self.buy_and_hold = False

    def next(self):
        if not self.buy_and_hold:
            self.buy_and_hold = True
            self.buy()

data = pd.read_csv('./data/binance_BTCUSDT_5m.csv')
data['datetime'] = pd.to_datetime(data['time'], unit='s')
data = data.drop(columns=['time'])

strategy = BuyAndHoldStrategy()
strategy.setdata(data)
trades = strategy.run()

th = TradeHistory(trades=trades)
print("Init Portfolio: ", INIT_PORTFOLIO)
print("Buy and Hold Strategy: ", th.study(cash=INIT_PORTFOLIO, sizer=1, commision=0.04, show_plot=False))


